-- -------------------------------------------------------------------------- --
-- PickPocketTimer by kunda                                                   --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- PickPocketTimer shows a small movable frame when you target a unit whose   --
-- pocket was picked by you. This frame shows a countdown which indicates     --
-- when this units' pocket can picked again.                                  --
--                                                                            --
-- This addon was mainly created as a little helper addon for the achievement --
-- 'Insane in the Membrane'.                                                  --
--                                                                            --
-- Textcolor:                                                                 --
-- red    = Pick Pocket NOT possible    = 480-180 seconds (8:00-3:00 minutes) --
-- yellow = Pick Pocket MAY BE possible = 179-  0 seconds (2:59-0:00 minutes) --
--                                                                            --
-- This Pocket-'Respawn Times' are based on empiric experience. The result of --
-- my tests varied from 314 (5:14) to 472 (7:52) seconds.                     --
-- Conclusion:                                                                --
--  After 300 seconds there is a chance to pick the same units' pocket again. --
--  After 480 seconds every units' pocket is 'respawned'.                     --
--  If you have other results, please let me know, thanks.                    --
--                                                                            --
-- -------------------------------------------------------------------------- --

--  ------------------------------------------------------------------------  --
--                                    STOP                                    --
--                   You must be a Rogue to enter this code.                  --
--                    If you are not a Rogue please leave.                    --
          if select(2, UnitClass("player")) ~= "ROGUE" then return end          
--  ------------------------------------------------------------------------  --

local date = date
local pairs = pairs
local time = time
local type = type
local floor = math.floor
local tinsert = table.insert
local wipe = table.wipe
local CreateFrame = CreateFrame
local ContainerIDToInventoryID = ContainerIDToInventoryID
local GetContainerNumFreeSlots = GetContainerNumFreeSlots
local GetCVar = GetCVar
local GetInventoryItemLink = GetInventoryItemLink
local GetItemFamily = GetItemFamily
local GetLootSlotInfo = GetLootSlotInfo
local GetLootSlotType = GetLootSlotType
local GetNumLootItems = GetNumLootItems
local GetTime = GetTime
local IsStealthed = IsStealthed
local UnitGUID = UnitGUID
local UnitIsDead = UnitIsDead

-- -------------------------------------------------------------------------- --

PickPocketTimer_Options = {} -- SavedVariable options table
local PickPocketTimer = CreateFrame("Frame", nil, UIParent)

local CountdownStatusMAYBE = 179 -- 179 seconds (2:59)
local CountdownStatusEND   = 480 -- 480 seconds (8:00)

local TimerDB = {}     -- table with Pick Pocket Duration
local lootTable = {}
local isPickPocket
local PickPocketGUID   -- Pick Pocket dstGUID (from COMBAT_LOG_EVENT_UNFILTERED)
local isNumLoot = 0    -- number of items in pocket loot
local oriNumLoot = 0   -- number of items in pocket loot at lootstart
local FreeBagSlots     -- availabe bag slots (only normal bags - GetItemFamily(bagLink) = 0)
local TargetGUID       -- current target GUID
local PlayerGUID       -- your GUID (set after event PLAYER_ENTERING_WORLD triggered)
local InCombat = false -- false = out of combat | true = in combat
local isAutoLoot       -- GetCVar("AutoLootDefault")
local numCoinSlots
local LootEventStart
local LootEventTimer
local MsgEventStart
local MsgEventTimer
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function DigitalTime(value)
	local dif = CountdownStatusEND - value
	local minutes = floor(dif/60)
	local seconds = dif-(minutes*60)
	if seconds < 10 then
		seconds = "0"..seconds
	end
	if dif <= CountdownStatusMAYBE then
		return "|cffffff00"..minutes..":"..seconds.."|r" -- yellow = Pick Pocket MAY BE possible
	else
		return "|cffff0000"..minutes..":"..seconds.."|r" -- red    = Pick Pocket NOT possible
	end
end

local function DurationCheck(currentTime)
	for k,v in pairs(TimerDB) do
		if (currentTime - v) >= CountdownStatusEND then
			TimerDB[k] = nil
			PickPocketTimer_Options.Data[k] = nil
		end
	end
end

local throttle = 0.3
local elapsed = 0
local function UpdateFrame(self, elap)
	elapsed = elapsed + elap
	if elapsed < throttle then return end
	elapsed = 0

	if InCombat then
		PickPocketTimer:Hide()
	elseif TimerDB[TargetGUID] then
		local currentTime = time()
		if UnitIsDead("target") then
			TimerDB[TargetGUID] = nil
			PickPocketTimer_Options.Data[TargetGUID] = nil
			DurationCheck(currentTime)
			PickPocketTimer:Hide()
		else
			local leftTime = currentTime - TimerDB[TargetGUID]
			if leftTime >= CountdownStatusEND then
				DurationCheck(currentTime)
				PickPocketTimer:Hide()
			else
				PickPocketTimer.Text:SetText(DigitalTime(leftTime))
			end
		end
	end
end

local function CheckTarget(currentTime)
	if InCombat then return end
	DurationCheck(currentTime)
	if TimerDB[TargetGUID] then
		PickPocketTimer:Show()
		PickPocketTimer.Text:SetText(DigitalTime(currentTime - TimerDB[TargetGUID]))
		PickPocketTimer:SetScript("OnUpdate", UpdateFrame)
	else
		PickPocketTimer:Hide()
		PickPocketTimer:SetScript("OnUpdate", nil)
	end
end

local function IsBag(bag)
	local invID = ContainerIDToInventoryID(bag)
	local bagLink = GetInventoryItemLink("player", invID)
	if bagLink then
		return bagLink
	else
		return false
	end
end

local function GetFreeBagSlots()
	local freeSlots = GetContainerNumFreeSlots(0)
	local containerBagType = 0
	for bag = 1, NUM_BAG_SLOTS do
		local bagLink = IsBag(bag)
		if bagLink then
			containerBagType = GetItemFamily(bagLink)
			if containerBagType == 0 then
				freeSlots = freeSlots + GetContainerNumFreeSlots(bag)
			end
		end
	end
	return freeSlots
end

local function SaveUnitToDB()
	local currentTime = time()
	TimerDB[PickPocketGUID] = currentTime
	PickPocketTimer_Options.Data[PickPocketGUID] = currentTime
	CheckTarget(currentTime)
	isNumLoot = 0
	isPickPocket = nil
end

local function SaveFramePos()
	PickPocketTimer_Options.posX = PickPocketTimer:GetLeft()
	PickPocketTimer_Options.posY = PickPocketTimer:GetTop()
end

local function Init()
	PickPocketTimer_Options.IssueMessage = nil -- old option
	if type(PickPocketTimer_Options.Data) ~= "table" then
		PickPocketTimer_Options.Data = {}
	else
		local currentTime = time()
		local paranoia = 0
		for k,v in pairs(PickPocketTimer_Options.Data) do
			paranoia = paranoia + 1
			if (currentTime - v) >= CountdownStatusEND then
				PickPocketTimer_Options.Data[k] = nil
			end
		end
		if paranoia > CountdownStatusEND then -- punk
			wipe(PickPocketTimer_Options.Data)
		end
		for k,v in pairs(PickPocketTimer_Options.Data) do
			TimerDB[k] = v
		end
	end

	PickPocketTimer:SetClampedToScreen(true)
	PickPocketTimer:SetMovable(true)
	PickPocketTimer:EnableMouse(true)
	PickPocketTimer:SetPoint("CENTER", UIParent, 0, 200)
	PickPocketTimer:SetFrameStrata("DIALOG")
	PickPocketTimer:SetWidth(75)
	PickPocketTimer:SetHeight(25)
	PickPocketTimer:Hide()
	PickPocketTimer:SetScript("OnMouseDown", function() PickPocketTimer:StartMoving() end)
	PickPocketTimer:SetScript("OnMouseUp", function() PickPocketTimer:StopMovingOrSizing() SaveFramePos() end)
	PickPocketTimer:SetScript("OnDragStop", function() PickPocketTimer:StopMovingOrSizing() end)

	local bgtexture = PickPocketTimer:CreateTexture(nil, "BACKGROUND")
	bgtexture:SetAllPoints(PickPocketTimer)
	bgtexture:SetTexture(0, 0, 0, 1)

	local button = CreateFrame("Button", nil, PickPocketTimer)
	button:SetFrameLevel(PickPocketTimer:GetFrameLevel()+1)
	button:SetWidth(25)
	button:SetHeight(25)
	button:SetPoint("LEFT", PickPocketTimer, "LEFT", -2, 0)
	button:SetScript("OnMouseDown", function() PickPocketTimer:StartMoving() end)
	button:SetScript("OnMouseUp", function() PickPocketTimer:StopMovingOrSizing() SaveFramePos() end)
	button:SetScript("OnDragStop", function() PickPocketTimer:StopMovingOrSizing() end)

	local normalTexture = button:CreateTexture(nil, "BACKGROUND")
	normalTexture:SetAllPoints(button)
	normalTexture:SetTexture("Interface\\Icons\\INV_Misc_Bag_11")
	button:SetNormalTexture(normalTexture)

	PickPocketTimer.Text = PickPocketTimer:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	PickPocketTimer.Text:SetPoint("LEFT", button, "RIGHT", 12, 0)

	if PickPocketTimer_Options.posX then
		PickPocketTimer:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", PickPocketTimer_Options.posX, PickPocketTimer_Options.posY)
	end

	LootEventTimer = CreateFrame("Frame", nil, UIParent)
	MsgEventTimer = CreateFrame("Frame", nil, UIParent)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local throttle = 0.1
local elapsed = 0
local function LootStop(self, elap)
	elapsed = elapsed + elap
	if elapsed < throttle then return end
	elapsed = 0

	if GetTime() > LootEventStart then
		PickPocketTimer:UnregisterEvent("LOOT_OPENED")
		PickPocketTimer:UnregisterEvent("LOOT_SLOT_CLEARED")
		PickPocketTimer:UnregisterEvent("LOOT_CLOSED")
		LootEventTimer:SetScript("OnUpdate", nil)
	end
end

local throttle = 0.1
local elapsed = 0
local function MsgStop(self, elap)
	elapsed = elapsed + elap
	if elapsed < throttle then return end
	elapsed = 0

	local LootFrame = LootFrame
	if LootFrame and LootFrame:IsShown() then
		MsgEventStart = GetTime() + 3
		return
	end

	if GetTime() > MsgEventStart then
		PickPocketTimer:UnregisterEvent("CHAT_MSG_LOOT")
		PickPocketTimer:UnregisterEvent("CHAT_MSG_MONEY")
		MsgEventTimer:SetScript("OnUpdate", nil)
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function StealthCheck()
	if IsStealthed() then
		PickPocketTimer:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	else
		PickPocketTimer:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function OnEvent(self, event, ...)
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		local _, eventName, _, sourceGUID, _, _, _, destGUID, _, _, _, spellId = ...
		if spellId ~= 921 then return end
		if sourceGUID ~= PlayerGUID then return end
		if not destGUID then return end
		if eventName == "SPELL_CAST_SUCCESS" then
			isPickPocket = true
			PickPocketGUID = destGUID
			LootEventStart = GetTime() + 1
			LootEventTimer:SetScript("OnUpdate", LootStop) -- to avoid unwanted errors while moving: if there is no LOOT* event in the next second unregister all LOOT* events
			PickPocketTimer:RegisterEvent("LOOT_OPENED")
			PickPocketTimer:RegisterEvent("LOOT_SLOT_CLEARED")
			PickPocketTimer:RegisterEvent("LOOT_CLOSED")
		end

	elseif event == "UPDATE_STEALTH" then
		StealthCheck()

	elseif event == "PLAYER_TARGET_CHANGED" then
		TargetGUID = UnitGUID("target")
		if isPickPocket and TargetGUID ~= PickPocketGUID then
			LootEventTimer:SetScript("OnUpdate", nil)
			PickPocketTimer:UnregisterEvent("LOOT_OPENED")
			PickPocketTimer:UnregisterEvent("LOOT_SLOT_CLEARED")
			PickPocketTimer:UnregisterEvent("LOOT_CLOSED")
			MsgEventTimer:SetScript("OnUpdate", nil)
			PickPocketTimer:UnregisterEvent("CHAT_MSG_LOOT")
			PickPocketTimer:UnregisterEvent("CHAT_MSG_MONEY")
			isPickPocket = nil
		end
		CheckTarget(time())
	elseif event == "PLAYER_REGEN_DISABLED" then
		InCombat = true
	elseif event == "PLAYER_REGEN_ENABLED" then
		InCombat = false
		TargetGUID = UnitGUID("target")
		if TargetGUID then
			CheckTarget(time())
		end

	elseif event == "LOOT_OPENED" then
		LootEventTimer:SetScript("OnUpdate", nil)

		-- LOOT_SLOT_NONE = 0;
		-- LOOT_SLOT_ITEM = 1;     -- CHAT_MSG_LOOT You receive loot: [Battered Junkbox].      0 0  0 9  0 false false
		-- LOOT_SLOT_MONEY = 2;    -- CHAT_MSG_MONEY You loot 69 Copper      0 0  0 3  0 false false
		-- LOOT_SLOT_CURRENCY = 3;
		-- Interface\Icons\INV_Box_01 Battered Junkbox 1 1 nil nil nil nil
		-- Interface\Icons\INV_Misc_Coin_06 43 Copper 0 1 nil nil nil nil
		isAutoLoot = GetCVar("AutoLootDefault")
		isNumLoot = GetNumLootItems()
		oriNumLoot = isNumLoot
		numCoinSlots = 0
		wipe(lootTable)
		for i = 1, isNumLoot do
			local _, item, quantity = GetLootSlotInfo(i)
			local slotType = GetLootSlotType(i)
			if slotType == 2 or slotType == 3 then -- don't know if it's possible to pick pocket currency
				numCoinSlots = numCoinSlots + 1
			end
			tinsert(lootTable, {item = item, quantity = quantity, slotType = slotType})
		end
		if isAutoLoot == "1" then
			if GetFreeBagSlots() >= (isNumLoot - numCoinSlots) then
				FreeBagSlots = true
			else
				FreeBagSlots = false
			end
		end

	elseif event == "LOOT_SLOT_CLEARED" then
		LootEventTimer:SetScript("OnUpdate", nil)
		isNumLoot = isNumLoot - 1

	elseif event == "LOOT_CLOSED" then
		LootEventTimer:SetScript("OnUpdate", nil)
		PickPocketTimer:UnregisterEvent("LOOT_OPENED")
		PickPocketTimer:UnregisterEvent("LOOT_SLOT_CLEARED")
		PickPocketTimer:UnregisterEvent("LOOT_CLOSED")

		if isNumLoot == 0 or (isAutoLoot == "1" and FreeBagSlots) then
			MsgEventTimer:SetScript("OnUpdate", nil)
			PickPocketTimer:UnregisterEvent("CHAT_MSG_LOOT")
			PickPocketTimer:UnregisterEvent("CHAT_MSG_MONEY")
			SaveUnitToDB()
		else
			local isItem, isCoin -- check: register always?
			for k, v in pairs(lootTable) do
				if v.slotType == 1 then
					isItem = true
				elseif v.slotType == 2 or v.slotType == 3 then
					isCoin = true
				end
			end
			if isItem or isCoin then
				MsgEventStart = GetTime() + 3
				MsgEventTimer:SetScript("OnUpdate", MsgStop)
				PickPocketTimer:RegisterEvent("CHAT_MSG_LOOT") -- the whole CHAT_MSG_* thing needs some improvement ...
				PickPocketTimer:RegisterEvent("CHAT_MSG_MONEY")
			end
		end

	elseif event == "CHAT_MSG_LOOT" then
		isNumLoot = isNumLoot - 1
		if isNumLoot <= 0 then
			MsgEventTimer:SetScript("OnUpdate", nil)
			PickPocketTimer:UnregisterEvent("CHAT_MSG_LOOT")
			PickPocketTimer:UnregisterEvent("CHAT_MSG_MONEY")
			if isNumLoot == 0 then
				SaveUnitToDB()
			end
		end
	elseif event == "CHAT_MSG_MONEY" then
		isNumLoot = isNumLoot - 1
		if isNumLoot <= 0 then
			MsgEventTimer:SetScript("OnUpdate", nil)
			PickPocketTimer:UnregisterEvent("CHAT_MSG_LOOT")
			PickPocketTimer:UnregisterEvent("CHAT_MSG_MONEY")
			if isNumLoot == 0 then
				SaveUnitToDB()
			end
		end

	elseif event == "PLAYER_ENTERING_WORLD" then
		PlayerGUID = UnitGUID("player")
		Init()
		StealthCheck()
		PickPocketTimer:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

PickPocketTimer:RegisterEvent("UPDATE_STEALTH")
PickPocketTimer:RegisterEvent("PLAYER_TARGET_CHANGED")
PickPocketTimer:RegisterEvent("PLAYER_REGEN_DISABLED")
PickPocketTimer:RegisterEvent("PLAYER_REGEN_ENABLED")
PickPocketTimer:RegisterEvent("PLAYER_ENTERING_WORLD")
PickPocketTimer:SetScript("OnEvent", OnEvent)